package com.rockymadden.jbrainhoney.cryptography;

import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

/** An abstract implementation of a Digest based upon the Java MessageDigest class. */
public abstract class MessageDigest implements Digest {
	private static final long serialVersionUID = 275923470972341235L;
	protected final Algorithm algorithm;
	protected final byte[] message;

	// Lazy attributes.
	private byte[] digest; // Immutable, does not need volatile.

	protected MessageDigest(final Builder builder) throws IllegalArgumentException {
		if(builder.algorithm == null) throw new IllegalArgumentException("Excpected algorithm to not be null.");
		else if(builder.message == null) throw new IllegalArgumentException("Excpected message to not be null.");
		else if(builder.message.length == 0)
			throw new IllegalArgumentException("Excpected message to have a length greater than 0.");

		algorithm = builder.algorithm;
		message = builder.message;
	}

	@Override
	public final Algorithm getAlgorithm() {
		return algorithm;
	}

	@Override
	public final byte[] getMessage() {
		return message;
	}

	@Override
	public final byte[] compute() throws CryptographyException {
		if(digest == null) synchronized(this) {
			if(digest == null) {
				try {
					java.security.MessageDigest messageDigest = java.security.MessageDigest.getInstance(algorithm.getName());
					messageDigest.update(message, 0, message.length);

					digest = messageDigest.digest();
				} catch(NoSuchAlgorithmException exception) {
					throw new CryptographyException(exception);
				}
			}
		}

		return digest;
	}

	@Override
	public final String computeAsString() {
		StringBuilder builder = new StringBuilder();
		BigInteger integer = new BigInteger(1, compute());

		builder.append(integer.toString(16));

		while(builder.length() < (algorithm.getBits()/4)) {
			builder.insert(0, "0");
		}

		return builder.toString();
	}

	@Override
	public final boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Digest)) return false;
		else return toString().equals(((Digest)object).toString());
	}

	@Override
	public final String toString() {
		return this.getClass().getSimpleName() + "(" + "algorithm=" + algorithm.toString() + ", " +"message=" + Arrays.toString(message) + ")";
	}

	@Override
	public final int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	/** An abstract builder of immutable MessageDigest instances. */
	protected abstract static class Builder implements Digest.Builder {
		protected final Algorithm algorithm;
		protected final byte[] message;

		protected Builder(final Algorithm algorithm, final byte[] message) {
			this.algorithm = algorithm;
			this.message = message;
		}

		protected Builder(final Algorithm algorithm, final String message) {
			this.algorithm = algorithm;

			if(message != null) this.message = message.getBytes();
			else this.message = new byte[0];
		}
	}
}