package com.rockymadden.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import com.rockymadden.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which adds one or more member enrollments to an existing group. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/AddGroupMembers">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class AddGroupMembersCommand implements Command {
	private final String groupId;
	private final List<String> enrollmentIds;

	@SuppressWarnings("synthetic-access")
	private AddGroupMembersCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.groupId == null) throw new IllegalArgumentException("Expected groupId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.groupId))
			throw new IllegalArgumentException("Expected groupId to be valid.");

		for(String enrollmentId : builder.enrollmentIds) {
			if(enrollmentId == null) throw new IllegalArgumentException("Expected enrollmentId to not be null.");
			else if(!EntityIdAttributeValidator.getInstance().isValid(enrollmentId))
				throw new IllegalArgumentException("Expected enrollmentId to be valid.");
		}

		groupId = builder.groupId;
		enrollmentIds = Collections.unmodifiableList(new ArrayList<String>(builder.enrollmentIds));
	}

	public String getGroupId() {
		return groupId;
	}

	public List<String> getEnrollmentIds() {
		return enrollmentIds;
	}

	/** A builder of immutable AddGroupMembersCommand instances. */
	public final static class Builder implements Command.Builder {
		private final String groupId;
		private final List<String> enrollmentIds;

		public <T extends Object> Builder(final String groupId, final List<T> enrollmentIds) {
			super();

			this.groupId = groupId;

			if(enrollmentIds == null) this.enrollmentIds = Collections.synchronizedList(new ArrayList<String>());
			else {
				List<String> enrollmentIdsCast = new ArrayList<String>(enrollmentIds.size());

				for(Object enrollmentId : enrollmentIds) {
					if(enrollmentId instanceof String || enrollmentId instanceof Integer)
						enrollmentIdsCast.add(enrollmentId.toString());
				}

				this.enrollmentIds = Collections.synchronizedList(enrollmentIdsCast);
			}
		}

		public <T extends Object> Builder(final Integer groupId, final List<T> enrollmentIds) {
			super();

			this.groupId = (groupId == null ? null : groupId.toString());

			if(enrollmentIds == null) this.enrollmentIds = Collections.synchronizedList(new ArrayList<String>());
			else {
				List<String> enrollmentIdsCast = new ArrayList<String>(enrollmentIds.size());

				for(Object enrollmentId : enrollmentIds) {
					if(enrollmentId instanceof String || enrollmentId instanceof Integer)
						enrollmentIdsCast.add(enrollmentId.toString());
				}

				this.enrollmentIds = Collections.synchronizedList(enrollmentIdsCast);
			}
		}

		public Builder(final String groupId, final String enrollmentId) {
			super();

			this.groupId = groupId;

			if(enrollmentId == null) enrollmentIds = Collections.synchronizedList(new ArrayList<String>());
			else enrollmentIds = Collections.synchronizedList(new ArrayList<String>(Arrays.asList(enrollmentId)));
		}

		public Builder(final String groupId, final Integer enrollmentId) {
			super();

			this.groupId = groupId;

			if(enrollmentId == null) enrollmentIds = Collections.synchronizedList(new ArrayList<String>());
			else enrollmentIds = Collections.synchronizedList(new ArrayList<String>(Arrays.asList(enrollmentId.toString())));
		}

		public Builder(final Integer groupId, final String enrollmentId) {
			super();

			this.groupId = (groupId == null ? null : groupId.toString());

			if(enrollmentId == null) enrollmentIds = Collections.synchronizedList(new ArrayList<String>());
			else enrollmentIds = Collections.synchronizedList(new ArrayList<String>(Arrays.asList(enrollmentId)));
		}

		public Builder(final Integer groupId, final Integer enrollmentId) {
			super();

			this.groupId = (groupId == null ? null : groupId.toString());

			if(enrollmentId == null) enrollmentIds = Collections.synchronizedList(new ArrayList<String>());
			else enrollmentIds = Collections.synchronizedList(new ArrayList<String>(Arrays.asList(enrollmentId.toString())));
		}

		public Builder addEnrollmentId(final String enrollmentId) {
			enrollmentIds.add(enrollmentId);

			return this;
		}

		public Builder addEnrollmentId(final Integer enrollmentId) {
			enrollmentIds.add((enrollmentId == null ? null : enrollmentId.toString()));

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public AddGroupMembersCommand build() {
			return new AddGroupMembersCommand(this);
		}
	}
}