package com.rockymadden.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import com.rockymadden.jbrainhoney.dlap.element.Course;

/**
 * A command which creates one or more courses. Automatically enrolls the calling user as the course owner. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateCourses">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class CreateCoursesCommand implements Command {
	private final List<Course> courses;

	@SuppressWarnings("synthetic-access")
	private CreateCoursesCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.courses == null || builder.courses.size() == 0)
			throw new IllegalArgumentException("Expected courses to not be null.");

		for(Course course : builder.courses) {
			if(course == null) throw new IllegalArgumentException("Expected course to not be null.");
			else if(course.getDomainId() == null || course.getProtection() == null || course.getStatus() == null
				|| course.getSchema() == null || course.getTitle() == null) {

				throw new IllegalArgumentException("Expected domainId, protection, status, schema, and title to all not be null.");
			}
		}

		courses = Collections.unmodifiableList(new ArrayList<Course>(builder.courses));
	}

	public List<Course> getCourses() {
		return courses;
	}

	/** A builder of immutable CreateCoursesCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Course> courses;

		public Builder(final List<Course> courses) {
			super();

			if(courses == null) this.courses = Collections.synchronizedList(new ArrayList<Course>());
			else this.courses = Collections.synchronizedList(new ArrayList<Course>(courses));
		}

		public Builder(final Course course) {
			super();

			if(course == null) courses = Collections.synchronizedList(new ArrayList<Course>());
			else courses = Collections.synchronizedList(new ArrayList<Course>(Arrays.asList(course)));
		}

		public Builder addCourse(final Course course) {
			courses.add(course);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public CreateCoursesCommand build() {
			return new CreateCoursesCommand(this);
		}
	}
}