package com.rockymadden.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import com.rockymadden.jbrainhoney.dlap.element.Domain;
import com.rockymadden.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which creates one or more domains and links them as children to the domain specified by parentId. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateDomains">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class CreateDomainsCommand implements Command {
	private final String parentId;
	private final List<Domain> domains;

	@SuppressWarnings("synthetic-access")
	private CreateDomainsCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.parentId == null) throw new IllegalArgumentException("Expected parentId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.parentId))
			throw new IllegalArgumentException("Expected parentId to be valid.");

		if(builder.domains == null || builder.domains.size() == 0)
			throw new IllegalArgumentException("Expected domains to not be null.");

		for(Domain domain : builder.domains) {
			if(domain == null) throw new IllegalArgumentException("Expected domain to not be null.");
			else if(domain.getName() == null || domain.getUserspace() == null)
				throw new IllegalArgumentException("Expected name and userspace to all not be null.");
		}

		parentId = builder.parentId;
		domains = Collections.unmodifiableList(new ArrayList<Domain>(builder.domains));
	}

	public String getParentId() {
		return parentId;
	}

	public List<Domain> getDomains() {
		return domains;
	}

	/** A builder of immutable CreateDomainsCommand instances. */
	public final static class Builder implements Command.Builder {
		private final String parentId;
		private final List<Domain> domains;

		public Builder(final String parentId, final List<Domain> domains) {
			super();

			this.parentId = parentId;

			if(domains == null) this.domains = Collections.synchronizedList(new ArrayList<Domain>());
			else this.domains = Collections.synchronizedList(new ArrayList<Domain>(domains));
		}

		public Builder(final Integer parentId, final List<Domain> domains) {
			super();

			this.parentId = (parentId == null ? null : parentId.toString());

			if(domains == null) this.domains = Collections.synchronizedList(new ArrayList<Domain>());
			else this.domains = Collections.synchronizedList(new ArrayList<Domain>(domains));
		}

		public Builder(final String parentId, final Domain domain) {
			super();

			this.parentId = parentId;

			if(domain == null) domains = Collections.synchronizedList(new ArrayList<Domain>());
			else domains = Collections.synchronizedList(new ArrayList<Domain>(Arrays.asList(domain)));
		}

		public Builder(final Integer parentId, final Domain domain) {
			super();

			this.parentId = (parentId == null ? null : parentId.toString());

			if(domain == null) domains = Collections.synchronizedList(new ArrayList<Domain>());
			else domains = Collections.synchronizedList(new ArrayList<Domain>(Arrays.asList(domain)));
		}

		public Builder addDomain(final Domain domain) {
			domains.add(domain);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public CreateDomainsCommand build() {
			return new CreateDomainsCommand(this);
		}
	}
}