package com.rockymadden.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import com.rockymadden.jbrainhoney.dlap.element.Section;

/**
 * A command creates one or more sections and enrolls the caller in the section as a section owner. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/CreateSections">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class CreateSectionsCommand implements Command {
	private final List<Section> sections;

	@SuppressWarnings("synthetic-access")
	private CreateSectionsCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.sections == null || builder.sections.size() == 0)
			throw new IllegalArgumentException("Expected sections to not be null.");

		for(Section section : builder.sections) {
			if(section == null) throw new IllegalArgumentException("Expected section to not be null.");
			else if(section.getCourseId() == null || section.getDomainId() == null || section.getTitle() == null
				|| section.getType() == null) {

				throw new IllegalArgumentException("Expected courseId, domainId, title, and type to all not be null.");
			}
		}

		sections = Collections.unmodifiableList(new ArrayList<Section>(builder.sections));
	}

	public List<Section> getSections() {
		return sections;
	}

	/** A builder of immutable CreateSectionsCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Section> sections;

		public Builder(final List<Section> sections) {
			super();

			if(sections == null) this.sections = Collections.synchronizedList(new ArrayList<Section>());
			else this.sections = Collections.synchronizedList(new ArrayList<Section>(sections));
		}

		public Builder(final Section section) {
			super();

			if(section == null) sections = Collections.synchronizedList(new ArrayList<Section>());
			else sections = Collections.synchronizedList(new ArrayList<Section>(Arrays.asList(section)));
		}

		public Builder addSection(final Section section) {
			sections.add(section);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public CreateSectionsCommand build() {
			return new CreateSectionsCommand(this);
		}
	}
}