package com.rockymadden.jbrainhoney.dlap.command;

import com.rockymadden.jbrainhoney.dlap.validate.UsernameAttributeValidator;
import com.rockymadden.jbrainhoney.dlap.validate.UserspaceAttributeValidator;

/**
 * A command which authenticates a user on the server and initiates a session state. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/Login">the BrainHoney DLAP API guide</a>} for this command.
 */
public final class LoginCommand implements Command, ObservableSessionCommand {
	private final String password;
	private final SessionObserverManager sessionObserverManager;
	private final String username;
	private final String userspace;

	@SuppressWarnings("synthetic-access")
	private LoginCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.password == null)
			throw new IllegalArgumentException("Expected password to not be null.");
		else if(builder.sessionObserverManager == null)
			throw new IllegalArgumentException("Expected sessionObserverManager to not be null.");
		else if(builder.username == null) throw new IllegalArgumentException("Expected username to not be null.");
		else if(!UsernameAttributeValidator.getInstance().isValid(builder.username))
			throw new IllegalArgumentException("Expected username to be valid.");
		else if(builder.userspace != null && !UserspaceAttributeValidator.getInstance().isValid(builder.userspace))
			throw new IllegalArgumentException("Expected userspace to be valid.");

		password = builder.password;
		sessionObserverManager = builder.sessionObserverManager;
		username = builder.username;
		userspace = builder.userspace;
	}

	public String getPassword() {
		return password;
	}

	public SessionObserverManager getSessionObserverManager() {
		return sessionObserverManager;
	}

	public String getUsername() {
		return username;
	}

	public String getUserspace() {
		return userspace;
	}

	@Override
	public LoginCommand addObserver(final SessionObserver observer) {
		sessionObserverManager.addObserver(observer);

		return this;
	}

	@Override
	public LoginCommand removeObserver(final SessionObserver observer) {
		sessionObserverManager.removeObserver(observer);

		return this;
	}

	/** A builder of immutable LoginCommand instances. */
	public final static class Builder implements Command.Builder, ObservableSessionCommand.Builder {
		private final String password;
		private final SessionObserverManager sessionObserverManager;
		private final String username;
		private String userspace;

		public Builder(final String username, final String password) {
			super();

			this.password = password;
			this.sessionObserverManager = new SessionObserverManager();
			this.username = username;
		}

		public Builder setUserspace(final String userspace) {
			this.userspace = userspace;

			return this;
		}

		@Override
		public Builder addObserver(final SessionObserver observer) {
			sessionObserverManager.addObserver(observer);

			return this;
		}

		@Override
		public Builder removeObserver(final SessionObserver observer) {
			sessionObserverManager.removeObserver(observer);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public LoginCommand build() {
			return new LoginCommand(this);
		}
	}
}