package com.rockymadden.jbrainhoney.dlap.command;

import com.rockymadden.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which updates the password question and answer for the specified user. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdatePasswordQuestionAnswer">the BrainHoney DLAP API guide</a>}
 * for this command.
 */
public final class UpdatePasswordQuestionAnswerCommand implements Command {
	private final String oldPassword;
	private final String passwordAnswer;
	private final String passwordQuestion;
	private final String userId;

	@SuppressWarnings("synthetic-access")
	private UpdatePasswordQuestionAnswerCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.passwordAnswer == null)
			throw new IllegalArgumentException("Expected passwordAnswer to not be null.");
		else if(builder.passwordAnswer.length() > 128)
			throw new IllegalArgumentException("Expected passwordAnswer to be less than or equal to 128 characters.");
		else if(builder.passwordQuestion == null)
			throw new IllegalArgumentException("Expected passwordQuestion to not be null.");
		else if(builder.passwordQuestion.length() > 256)
			throw new IllegalArgumentException("Expected passwordQuestion to be less than or equal to 256 characters.");
		else if(builder.userId == null) throw new IllegalArgumentException("Expected userId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.userId))
			throw new IllegalArgumentException("Expected userId to be valid.");

		oldPassword = builder.oldPassword;
		passwordAnswer = builder.passwordAnswer;
		passwordQuestion = builder.passwordQuestion;
		userId = builder.userId;
	}

	public String getOldPassword() {
		return oldPassword;
	}

	public String getPasswordAnswer() {
		return passwordAnswer;
	}

	public String getPasswordQuestion() {
		return passwordQuestion;
	}

	public String getUserId() {
		return userId;
	}

	/** A builder of immutable UpdatePasswordQuestionAnswerCommand instances. */
	public final static class Builder implements Command.Builder {
		private String oldPassword;
		private final String passwordAnswer;
		private final String passwordQuestion;
		private final String userId;

		public Builder(final String userId, final String passwordQuestion, final String passwordAnswer) {
			super();

			this.userId = userId;
			this.passwordQuestion = passwordQuestion;
			this.passwordAnswer = passwordAnswer;
		}

		public Builder(final Integer userId, final String passwordQuestion, final String passwordAnswer) {
			super();

			this.userId = (userId == null ? null : userId.toString());
			this.passwordQuestion = passwordQuestion;
			this.passwordAnswer = passwordAnswer;
		}

		public Builder setOldPassword(final String oldPassword) {
			this.oldPassword = oldPassword;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public UpdatePasswordQuestionAnswerCommand build() {
			return new UpdatePasswordQuestionAnswerCommand(this);
		}
	}
}