package com.rockymadden.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import com.rockymadden.jbrainhoney.dlap.element.Right;

/**
 * A command which updates the rights (flags) granted to the specified user (actorId) for the domain specified by
 * entityId. See the {@link <a href="http://gls.agilix.com/Docs/Command/UpdateRights">the BrainHoney DLAP API guide</a>}
 * for this command.
 */
public final class UpdateRightsCommand implements Command {
	private final List<Right> rights;

	@SuppressWarnings("synthetic-access")
	private UpdateRightsCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.rights == null || builder.rights.size() == 0)
			throw new IllegalArgumentException("Expected rights to not be null.");

		for(Right right : builder.rights) {
			if(right == null) throw new IllegalArgumentException("Expected right to not be null.");
			else if(right.getActorId() == null || right.getEntityId() == null || right.getFlags() == null)
				throw new IllegalArgumentException("Expected actorId, entityId, and flags to not be null.");
		}

		rights = Collections.unmodifiableList(new ArrayList<Right>(builder.rights));
	}

	public List<Right> getRights() {
		return rights;
	}

	/** A builder of immutable UpdateRightsCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Right> rights;

		public Builder(final List<Right> rights) {
			super();

			if(rights == null) this.rights = Collections.synchronizedList(new ArrayList<Right>());
			else this.rights = Collections.synchronizedList(new ArrayList<Right>(rights));
		}

		public Builder(final Right right) {
			super();

			if(right == null) rights = Collections.synchronizedList(new ArrayList<Right>());
			else rights = Collections.synchronizedList(new ArrayList<Right>(Arrays.asList(right)));
		}

		public Builder addRight(final Right right) {
			rights.add(right);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public UpdateRightsCommand build() {
			return new UpdateRightsCommand(this);
		}
	}
}