package com.rockymadden.jbrainhoney.dlap.element;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.EnumSet;
import java.util.List;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import com.rockymadden.jbrainhoney.compare.NullComparatorUtility;
import com.rockymadden.jbrainhoney.dlap.enumeration.*;
import com.rockymadden.jbrainhoney.dlap.validate.EntityIdAttributeValidator;
import com.rockymadden.jbrainhoney.dlap.validate.GuidAttributeValidator;
import com.rockymadden.jbrainhoney.dlap.validate.ReferenceAttributeValidator;
import com.rockymadden.jbrainhoney.dlap.validate.VersionAttributeValidator;
import com.rockymadden.jbrainhoney.time.DateTimeUtility;
import org.joda.time.DateTime;

/** A course. */
public final class Course implements XmlElement, Class, Entity, Partition, Comparable<Course> {
	private static final long serialVersionUID = -24405567425317132L;
	private final CopyAction action;
	private final String baseId;
	private final String courseId;
	private final DateTime creationDate;
	private final Element data;
	private final Integer days;
	private final String domainId;
	private final DateTime endDate;
	private final String guid;
	private final Long protection;
	private final String reference;
	private final EnrollmentStatus status;
	private final CourseSchema schema;
	private final DateTime startDate;
	private final String term;
	private final String title;
	private final CourseType type;
	private final Integer version;

	// Lazy attributes.
	private List<DigitalCourseProtection> digitalCourseProtections;

	@SuppressWarnings("synthetic-access")
	private Course(final Builder builder) throws IllegalArgumentException {
		Element xmlElement = null;

		if(builder.baseId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.baseId))
			throw new IllegalArgumentException("Expected baseId to be valid.");
		else if(builder.courseId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.courseId))
			throw new IllegalArgumentException("Expected courseId to be valid.");

		if(builder.data != null && builder.data.length() > 0) {
			nu.xom.Builder xmlBuilder = null;
			Document xmlDocument = null;

			try {
				xmlBuilder = new nu.xom.Builder();
				xmlDocument = xmlBuilder.build(builder.data, null);
			} catch(Exception exception) {
				throw new IllegalArgumentException("Expected data to be well formed XML.");
			}

			if(!xmlDocument.getRootElement().getLocalName().equals("data"))
				throw new IllegalArgumentException("Expected data root element.");

			xmlElement = (Element)xmlDocument.getRootElement().copy();
		}

		if(builder.days != null && builder.days < 0)
			throw new IllegalArgumentException("Expected days to be greater than or equal to 0.");
		else if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.guid != null && !GuidAttributeValidator.getInstance().isValid(builder.guid))
			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(builder.protection != null && builder.protection < 0)
			throw new IllegalArgumentException("Expected protection to be 0 or greater.");
		else if(builder.reference != null && !ReferenceAttributeValidator.getInstance().isValid(builder.reference))
			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(builder.term != null && builder.term.length() > 128)
			throw new IllegalArgumentException("Expected term to be less than 128 characters.");
		else if(builder.title != null && builder.title.length() > 128)
			throw new IllegalArgumentException("Expected title to be less than 128 characters.");
		else if(builder.version != null && !VersionAttributeValidator.getInstance().isValid(builder.version))
			throw new IllegalArgumentException("Expected version to be valid.");

		action = builder.action;
		baseId = builder.baseId;
		courseId = builder.courseId;
		creationDate = builder.creationDate;
		data = xmlElement;
		days = builder.days;
		domainId = builder.domainId;
		endDate = builder.endDate;
		guid = builder.guid;
		protection = builder.protection;
		reference = builder.reference;
		status = builder.status;
		schema = builder.schema;
		startDate = builder.startDate;
		term = builder.term;
		title = builder.title;
		type = builder.type;
		version = builder.version;
	}

	public Course(final nu.xom.Element element) throws IllegalArgumentException {
		if(!element.getQualifiedName().equals("course"))
			throw new IllegalArgumentException("Expected course XML element.");
		else if(element.getAttribute("baseid") != null && element.getAttribute("baseid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("baseid").getValue()))

			throw new IllegalArgumentException("Expected baseId to be valid.");
		else if(element.getAttribute("courseid") != null && element.getAttribute("courseid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("courseid").getValue()))

			throw new IllegalArgumentException("Expected courseId to be valid.");
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("id").getValue()))

			throw new IllegalArgumentException("Expected courseId to be valid.");
		else if(element.getAttribute("days") != null && element.getAttribute("days").getValue().length() > 0
			&& Integer.parseInt(element.getAttribute("days").getValue()) < 0)

			throw new IllegalArgumentException("Expected days to be greater than or equal to 0.");
		else if(element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("domainid").getValue()))

			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0
			&& !GuidAttributeValidator.getInstance().isValid(element.getAttribute("guid").getValue()))

			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(element.getAttribute("protection") != null && element.getAttribute("protection").getValue().length() > 0
			&& Integer.parseInt(element.getAttribute("protection").getValue()) < 0)

			throw new IllegalArgumentException("Expected protection to be 0 or greater.");
		else if(element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0
			&& !ReferenceAttributeValidator.getInstance().isValid(element.getAttribute("reference").getValue()))

			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(element.getAttribute("term") != null && element.getAttribute("term").getValue().length() > 128)
			throw new IllegalArgumentException("Expected term to be less than 128 characters.");
		else if(element.getAttribute("title") != null && element.getAttribute("title").getValue().length() > 128)
			throw new IllegalArgumentException("Expected title to be less than 128 characters.");
		else if(element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0
			&& !VersionAttributeValidator.getInstance().isValid(element.getAttribute("version").getValue()))

			throw new IllegalArgumentException("Expected version to be valid.");

		action = (element.getAttribute("action") != null && element.getAttribute("action").getValue().length() > 0 ?
			CopyAction.valueOfValue(element.getAttribute("action").getValue()) : null);
		baseId = (element.getAttribute("baseid") != null && element.getAttribute("baseid").getValue().length() > 0
			&& !element.getAttribute("baseid").getValue().equals("0") ?

			element.getAttribute("baseid").getValue() : null); // Check for 0.

		// Referenced as courseid and id.
		if(element.getAttribute("courseid") != null && element.getAttribute("courseid").getValue().length() > 0)
			courseId = element.getAttribute("courseid").getValue();
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0)
			courseId = element.getAttribute("id").getValue();
		else courseId = null;

		creationDate = (element.getAttribute("creationdate") != null && element.getAttribute("creationdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("creationdate").getValue()) : null);
		data = (element.getFirstChildElement("data") != null ?
			(Element)element.getFirstChildElement("data").copy() : null);
		days = (element.getAttribute("days") != null && element.getAttribute("days").getValue().length() > 0 ?
			Integer.parseInt(element.getAttribute("days").getValue()) : null);
		domainId = (element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0 ?
			element.getAttribute("domainid").getValue() : null);
		endDate = (element.getAttribute("enddate") != null && element.getAttribute("enddate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("enddate").getValue()) : null);
		guid = (element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0 ?
			element.getAttribute("guid").getValue() : null);
		protection = (element.getAttribute("protection") != null && element.getAttribute("protection").getValue().length() > 0 ?
			Long.parseLong(element.getAttribute("protection").getValue()) : null);
		reference = (element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0 ?
			element.getAttribute("reference").getValue() : null);
		status = (element.getAttribute("status") != null && element.getAttribute("status").getValue().length() > 0 ?
			EnrollmentStatus.valueOfValue(Integer.parseInt(element.getAttribute("status").getValue())) : null);
		schema = (element.getAttribute("schema") != null && element.getAttribute("schema").getValue().length() > 0 ?
			CourseSchema.valueOfValue(Integer.parseInt(element.getAttribute("schema").getValue())) : null);
		startDate = (element.getAttribute("startdate") != null && element.getAttribute("startdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("startdate").getValue()) : null);
		term = (element.getAttribute("term") != null && element.getAttribute("term").getValue().length() > 0 ?
			element.getAttribute("term").getValue() : null);
		title = (element.getAttribute("title") != null && element.getAttribute("title").getValue().length() > 0 ?
			element.getAttribute("title").getValue() : null);
		type = (element.getAttribute("type") != null && element.getAttribute("type").getValue().length() > 0 ?
			CourseType.valueOfValue(element.getAttribute("type").getValue()) : null);
		version = (element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0 ?
			Integer.parseInt(element.getAttribute("version").getValue()) : null);
	}

	@Override
	public int compareTo(final Course course) throws NullPointerException {
		if(course == null) throw new NullPointerException("Expected course to not be null.");

		return NullComparatorUtility.compareIgnoreNullAndCase(this.courseId, course.getCourseId());
	}

	public CopyAction getAction() {
		return action;
	}

	public String getBaseId() {
		return baseId;
	}

	public String getCourseId() {
		return courseId;
	}

	public DateTime getCreationDate() {
		return creationDate;
	}

	public Element getData() {
		return (data != null ? (Element)data.copy() : null);
	}

	public Integer getDays() {
		return days;
	}

	public String getDomainId() {
		return domainId;
	}

	public DateTime getEndDate() {
		return endDate;
	}

	public String getGuid() {
		return guid;
	}

	public Long getProtection() {
		return protection;
	}

	public String getReference() {
		return reference;
	}

	public EnrollmentStatus getStatus() {
		return status;
	}

	public CourseSchema getSchema() {
		return schema;
	}

	public DateTime getStartDate() {
		return startDate;
	}

	public String getTerm() {
		return term;
	}

	public String getTitle() {
		return title;
	}

	public CourseType getType() {
		return type;
	}

	public Integer getVersion() {
		return version;
	}

	// ** Convenience method. Returns null if protection is null. */
	public List<DigitalCourseProtection> getDigitalCourseProtections() {
		if(protection != null) {
			if(digitalCourseProtections == null) synchronized(this) {
				if(digitalCourseProtections == null)
					digitalCourseProtections = Collections.unmodifiableList(
						new ArrayList<DigitalCourseProtection>(
							BitwiseUtility.toEnumeration(protection, EnumSet.allOf(DigitalCourseProtection.class))
						)
					);
			}

			return digitalCourseProtections;
		}

		return null;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Course)) return false;
		else return toString().equals(((Course)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(Course.class.getSimpleName() + "(");
		if(action != null) stringBuilder.append("action=" + action.getValue() + ", ");
		if(baseId != null) stringBuilder.append("baseId=" + baseId + ", ");
		if(courseId != null) stringBuilder.append("courseId=" + courseId + ", ");
		if(creationDate != null) stringBuilder.append("creationDate=" + creationDate.toString() + ", ");
		if(data != null) stringBuilder.append("data=" + data.toXML() + ", ");
		if(days != null) stringBuilder.append("days=" + days.toString() + ", ");
		if(domainId != null) stringBuilder.append("domainId=" + domainId + ", ");
		if(endDate != null) stringBuilder.append("endDate=" + endDate.toString() + ", ");
		if(guid != null) stringBuilder.append("guid=" + guid + ", ");
		if(protection != null) stringBuilder.append("protection=" + protection.toString() + ", ");
		if(reference != null) stringBuilder.append("reference=" + reference + ", ");
		if(status != null) stringBuilder.append("status=" + status.getValue().toString() + ", ");
		if(schema != null) stringBuilder.append("schema=" + schema.getValue().toString() + ", ");
		if(startDate != null) stringBuilder.append("startDate=" + startDate.toString() + ", ");
		if(term != null) stringBuilder.append("term=" + term + ", ");
		if(title != null) stringBuilder.append("title=" + title + ", ");
		if(type != null) stringBuilder.append("type=" + type.getValue().toString() + ", ");
		if(version != null) stringBuilder.append("version=" + version.toString() + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("course");

		if(action != null) element.addAttribute(new Attribute("action", action.getValue()));
		if(baseId != null) element.addAttribute(new Attribute("baseid", baseId));
		if(courseId != null) element.addAttribute(new Attribute("courseid", courseId));
		if(creationDate != null) element.addAttribute(new Attribute("creationdate", creationDate.toString()));
		if(data != null) element.appendChild(data.copy());
		if(days != null) element.addAttribute(new Attribute("days", days.toString()));
		if(domainId != null) element.addAttribute(new Attribute("domainid", domainId));
		if(endDate != null) element.addAttribute(new Attribute("enddate", endDate.toString()));
		if(guid != null) element.addAttribute(new Attribute("guid", guid));
		if(protection != null) element.addAttribute(new Attribute("protection", protection.toString()));
		if(reference != null) element.addAttribute(new Attribute("reference", reference));
		if(status != null) element.addAttribute(new Attribute("status", status.getValue().toString()));
		if(schema != null) element.addAttribute(new Attribute("schema", schema.getValue().toString()));
		if(startDate != null) element.addAttribute(new Attribute("startdate", startDate.toString()));
		if(term != null) element.addAttribute(new Attribute("term", term));
		if(title != null) element.addAttribute(new Attribute("title", title));
		if(type != null) element.addAttribute(new Attribute("type", type.getValue().toString()));
		if(version != null) element.addAttribute(new Attribute("version", version.toString()));

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable Course elements. */
	public final static class Builder implements XmlElement.Builder {
		private CopyAction action;
		private String baseId;
		private String courseId;
		private DateTime creationDate;
		private String data;
		private Integer days;
		private String domainId;
		private DateTime endDate;
		private String guid;
		private Long protection;
		private String reference;
		private EnrollmentStatus status;
		private CourseSchema schema;
		private DateTime startDate;
		private String term;
		private String title;
		private CourseType type;
		private Integer version;

		public Builder() {

		}

		public Builder setAction(final CopyAction action) {
			this.action = action;

			return this;
		}

		public Builder setAction(final String action) {
			this.action = (action == null ? null : CopyAction.valueOfValue(action));

			return this;
		}

		public Builder setBaseId(final String baseId) {
			this.baseId = baseId;

			return this;
		}

		public Builder setBaseId(final Integer baseId) {
			this.baseId = (baseId == null ? null : baseId.toString());

			return this;
		}

		public Builder setCourseId(final String courseId) {
			this.courseId = courseId;

			return this;
		}

		public Builder setCourseId(final Integer courseId) {
			this.courseId = (courseId == null ? null : courseId.toString());

			return this;
		}

		public Builder setCreationDate(final Date creationDate) {
			this.creationDate = (creationDate == null ? null : new DateTime(creationDate));

			return this;
		}

		public Builder setCreationDate(final DateTime creationDate) {
			this.creationDate = creationDate;

			return this;
		}

		public Builder setCreationDate(final DateBoundary creationDate) {
			this.creationDate = creationDate.getValue();

			return this;
		}

		public Builder setData(final String data) {
			this.data = data;

			return this;
		}

		public Builder setData(final Element data) {
			this.data = (data == null ? null : data.toXML());

			return this;
		}

		public Builder setDays(final Integer days) {
			this.days = days;

			return this;
		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setEndDate(final Date endDate) {
			this.endDate = (endDate == null ? null : new DateTime(endDate));

			return this;
		}

		public Builder setEndDate(final DateTime endDate) {
			this.endDate = endDate;

			return this;
		}

		public Builder setEndDate(final DateBoundary endDate) {
			this.endDate = endDate.getValue();

			return this;
		}

		public Builder setGuid(final String guid) {
			this.guid = guid;

			return this;
		}

		public Builder setProtection(final Long protection) {
			this.protection = protection;

			return this;
		}

		public Builder setProtection(final DigitalCourseProtection protection) {
			this.protection = protection.getValue();

			return this;
		}

		public Builder setReference(final String reference) {
			this.reference = reference;

			return this;
		}

		public Builder setStatus(final EnrollmentStatus status) {
			this.status = status;

			return this;
		}

		public Builder setStatus(final Integer status) {
			this.status = (status == null ? null : EnrollmentStatus.valueOfValue(status));

			return this;
		}

		public Builder setSchema(final CourseSchema schema) {
			this.schema = schema;

			return this;
		}

		public Builder setSchema(final Integer schema) {
			this.schema = (schema == null ? null : CourseSchema.valueOfValue(schema));

			return this;
		}

		public Builder setStartDate(final Date startDate) {
			this.startDate = (startDate == null ? null : new DateTime(startDate));

			return this;
		}

		public Builder setStartDate(final DateTime startDate) {
			this.startDate = startDate;

			return this;
		}

		public Builder setStartDate(final DateBoundary startDate) {
			this.startDate = startDate.getValue();

			return this;
		}

		public Builder setTerm(final String term) {
			this.term = term;

			return this;
		}

		public Builder setTitle(final String title) {
			this.title = title;

			return this;
		}

		public Builder setType(final CourseType type) {
			this.type = type;

			return this;
		}

		public Builder setType(final String type) {
			this.type = (type == null ? null : CourseType.valueOfValue(type));

			return this;
		}

		public Builder setVersion(final Integer version) {
			this.version = version;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public Course build() {
			return new Course(this);
		}
	}
}