package com.rockymadden.jbrainhoney.dlap.element;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.EnumSet;
import java.util.List;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import com.rockymadden.jbrainhoney.compare.NullComparatorUtility;
import com.rockymadden.jbrainhoney.dlap.enumeration.DateBoundary;
import com.rockymadden.jbrainhoney.dlap.enumeration.EnrollmentStatus;
import com.rockymadden.jbrainhoney.dlap.enumeration.FlagSchema;
import com.rockymadden.jbrainhoney.dlap.enumeration.RightFlag;
import com.rockymadden.jbrainhoney.dlap.validate.EntityIdAttributeValidator;
import com.rockymadden.jbrainhoney.dlap.validate.GuidAttributeValidator;
import com.rockymadden.jbrainhoney.dlap.validate.ReferenceAttributeValidator;
import com.rockymadden.jbrainhoney.dlap.validate.VersionAttributeValidator;
import com.rockymadden.jbrainhoney.time.DateTimeUtility;
import org.joda.time.DateTime;

/** An enrollment. */
public final class Enrollment implements XmlElement, Comparable<Enrollment> {
	private static final long serialVersionUID = -66750546090811243L;
	private final DateTime creationDate;
	private final Element data;
	private final String domainId;
	private final DateTime endDate;
	private final String enrollmentId;
	private final String entityId;
	private final Long flags;
	private final String guid;
	private final EnrollmentStatus status;
	private final DateTime startDate;
	private final String reference;
	private final FlagSchema schema;
	private final String userId;
	private final Integer version;

	// Lazy attributes.
	private List<RightFlag> rightFlags;

	@SuppressWarnings("synthetic-access")
	private Enrollment(final Builder builder) throws IllegalArgumentException {
		Element xmlElement = null;

		if(builder.data != null && builder.data.length() > 0) {
			nu.xom.Builder xmlBuilder = null;
			Document xmlDocument = null;

			try {
				xmlBuilder = new nu.xom.Builder();
				xmlDocument = xmlBuilder.build(builder.data, null);
			} catch(Exception exception) {
				throw new IllegalArgumentException("Expected data to be well formed XML.");
			}

			if(!xmlDocument.getRootElement().getLocalName().equals("data"))
				throw new IllegalArgumentException("Expected data root element.");

			xmlElement = (Element)xmlDocument.getRootElement().copy();
		}

		if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.enrollmentId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.enrollmentId))
			throw new IllegalArgumentException("Expected enrollmentId to be valid.");
		else if(builder.entityId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.entityId))
			throw new IllegalArgumentException("Expected entityId to be valid.");
		else if(builder.guid != null && !GuidAttributeValidator.getInstance().isValid(builder.guid))
			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(builder.reference != null && !ReferenceAttributeValidator.getInstance().isValid(builder.reference))
			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(builder.userId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.userId))
			throw new IllegalArgumentException("Expected userId to be valid.");
		else if(builder.version != null && !VersionAttributeValidator.getInstance().isValid(builder.version))
			throw new IllegalArgumentException("Expected version to be valid.");

		creationDate = builder.creationDate;
		data = xmlElement;
		domainId = builder.domainId;
		endDate = builder.endDate;
		enrollmentId = builder.enrollmentId;
		entityId = builder.entityId;
		flags = builder.flags;
		guid = builder.guid;
		status = builder.status;
		startDate = builder.startDate;
		reference = builder.reference;
		schema = builder.schema;
		userId = builder.userId;
		version = builder.version;
	}

	public Enrollment(final nu.xom.Element element) throws IllegalArgumentException {
		if(!element.getQualifiedName().equals("enrollment"))
			throw new IllegalArgumentException("Expected enrollment XML element.");
		else if(element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("domainid").getValue()))

			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(element.getAttribute("enrollmentid") != null && element.getAttribute("enrollmentid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("enrollmentid").getValue()))

			throw new IllegalArgumentException("Expected enrollmentId to be valid.");
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("id").getValue()))

			throw new IllegalArgumentException("Expected enrollmentId to be valid.");
		else if(element.getAttribute("entityid") != null && element.getAttribute("entityid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("entityid").getValue()))

			throw new IllegalArgumentException("Expected entityId to be valid.");
		else if(element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0
			&& !GuidAttributeValidator.getInstance().isValid(element.getAttribute("guid").getValue()))

			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0
			&& !ReferenceAttributeValidator.getInstance().isValid(element.getAttribute("reference").getValue()))

			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(element.getAttribute("userid") != null && element.getAttribute("userid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("userid").getValue()))

			throw new IllegalArgumentException("Expected userId to be valid.");
		else if(element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0
			&& !VersionAttributeValidator.getInstance().isValid(element.getAttribute("version").getValue()))

			throw new IllegalArgumentException("Expected parentId to be valid.");

		creationDate = (element.getAttribute("creationdate") != null && element.getAttribute("creationdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("creationdate").getValue()) : null);
		data = (element.getFirstChildElement("data") != null ?
			(Element)element.getFirstChildElement("data").copy() : null);
		domainId = (element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0 ?
			element.getAttribute("domainid").getValue() : null);
		endDate = (element.getAttribute("enddate") != null && element.getAttribute("enddate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("enddate").getValue()) : null);

		// Referenced as enrollmentid and id.
		if(element.getAttribute("enrollmentid") != null && element.getAttribute("enrollmentid").getValue().length() > 0)
			enrollmentId = element.getAttribute("enrollmentid").getValue();
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0)
			enrollmentId = element.getAttribute("id").getValue();
		else enrollmentId = null;

		entityId = (element.getAttribute("entityid") != null && element.getAttribute("entityid").getValue().length() > 0 ?
			element.getAttribute("entityid").getValue() : null);
		flags = (element.getAttribute("flags") != null && element.getAttribute("flags").getValue().length() > 0 ?
			Long.parseLong(element.getAttribute("flags").getValue()) : null);
		guid = (element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0 ?
			element.getAttribute("guid").getValue() : null);
		status = (element.getAttribute("status") != null && element.getAttribute("status").getValue().length() > 0 ?
			EnrollmentStatus.valueOfValue(Integer.parseInt(element.getAttribute("status").getValue())) : null);
		startDate = (element.getAttribute("startdate") != null && element.getAttribute("startdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("startdate").getValue()) : null);
		reference = (element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0 ?
			element.getAttribute("reference").getValue() : null);
		schema = (element.getAttribute("schema") != null && element.getAttribute("schema").getValue().length() > 0 ?
			FlagSchema.valueOfValue(Integer.parseInt(element.getAttribute("schema").getValue())) : null);
		userId = (element.getAttribute("userid") != null && element.getAttribute("userid").getValue().length() > 0 ?
			element.getAttribute("userid").getValue() : null);
		version = (element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0 ?
			Integer.parseInt(element.getAttribute("version").getValue()) : null);
	}

	@Override
	public int compareTo(final Enrollment enrollment) throws NullPointerException {
		if(enrollment == null) throw new NullPointerException("Expected enrollment to not be null.");

		return NullComparatorUtility.compareIgnoreNullAndCase(this.enrollmentId, enrollment.getEnrollmentId());
	}

	public DateTime getCreationDate() {
		return creationDate;
	}

	public Element getData() {
		return (data != null ? (Element)data.copy() : null);
	}

	public String getDomainId() {
		return domainId;
	}

	public DateTime getEndDate() {
		return endDate;
	}

	public String getEnrollmentId() {
		return enrollmentId;
	}

	public String getEntityId() {
		return entityId;
	}

	public Long getFlags() {
		return flags;
	}

	public String getGuid() {
		return guid;
	}

	public EnrollmentStatus getStatus() {
		return status;
	}

	public DateTime getStartDate() {
		return startDate;
	}

	public String getReference() {
		return reference;
	}

	public FlagSchema getSchema() {
		return schema;
	}

	public String getUserId() {
		return userId;
	}

	public Integer getVersion() {
		return version;
	}

	// ** Convenience method. Returns null if flags is null. */
	public List<RightFlag> getRightFlags() {
		if(flags != null) {
			if(rightFlags == null) synchronized(this) {
				if(rightFlags == null)
					rightFlags = Collections.unmodifiableList(
						new ArrayList<RightFlag>(
							BitwiseUtility.toEnumeration(flags, EnumSet.allOf(RightFlag.class))
						)
					);
			}

			return rightFlags;
		}

		return null;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Enrollment)) return false;
		else return toString().equals(((Enrollment)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(Enrollment.class.getSimpleName() + "(");
		if(creationDate != null) stringBuilder.append("creationDate=" + creationDate.toString() + ", ");
		if(data != null) stringBuilder.append("data=" + data.toXML() + ", ");
		if(domainId != null) stringBuilder.append("domainId=" + domainId + ", ");
		if(endDate != null) stringBuilder.append("endDate=" + endDate.toString() + ", ");
		if(enrollmentId != null) stringBuilder.append("enrollmentId=" + enrollmentId + ", ");
		if(entityId != null) stringBuilder.append("entityId=" + entityId + ", ");
		if(flags != null) stringBuilder.append("flags=" + flags.toString() + ", ");
		if(guid != null) stringBuilder.append("guid=" + guid + ", ");
		if(status != null) stringBuilder.append("status=" + status.getValue().toString() + ", ");
		if(startDate != null) stringBuilder.append("startDate=" + startDate.toString() + ", ");
		if(reference != null) stringBuilder.append("reference=" + reference + ", ");
		if(schema != null) stringBuilder.append("schema=" + schema.getValue().toString() + ", ");
		if(userId != null) stringBuilder.append("userId=" + userId + ", ");
		if(version != null) stringBuilder.append("version=" + version.toString() + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("enrollment");

		if(creationDate != null) element.addAttribute(new Attribute("creationdate", creationDate.toString()));
		if(data != null) element.appendChild(data.copy());
		if(domainId != null) element.addAttribute(new Attribute("domainid", domainId));
		if(endDate != null) element.addAttribute(new Attribute("enddate", endDate.toString()));
		if(enrollmentId != null) element.addAttribute(new Attribute("enrollmentid", enrollmentId));
		if(entityId != null) element.addAttribute(new Attribute("entityid", entityId));
		if(flags != null) element.addAttribute(new Attribute("flags", flags.toString()));
		if(guid != null) element.addAttribute(new Attribute("guid", guid));
		if(status != null) element.addAttribute(new Attribute("status", status.getValue().toString()));
		if(startDate != null) element.addAttribute(new Attribute("startdate", startDate.toString()));
		if(reference != null) element.addAttribute(new Attribute("reference", reference));
		if(schema != null) element.addAttribute(new Attribute("schema", schema.getValue().toString()));
		if(userId != null) element.addAttribute(new Attribute("userid", userId));
		if(version != null) element.addAttribute(new Attribute("version", version.toString()));

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable Enrollment elements. */
	public final static class Builder implements XmlElement.Builder {
		private DateTime creationDate;
		private String data;
		private String domainId;
		private DateTime endDate;
		private String enrollmentId;
		private String entityId;
		private Long flags;
		private String guid;
		private EnrollmentStatus status;
		private DateTime startDate;
		private String reference;
		private FlagSchema schema;
		private String userId;
		private Integer version;

		public Builder() {

		}

		public Builder setCreationDate(final Date creationDate) {
			this.creationDate = (creationDate == null ? null : new DateTime(creationDate));

			return this;
		}

		public Builder setCreationDate(final DateTime creationDate) {
			this.creationDate = creationDate;

			return this;
		}

		public Builder setCreationDate(final DateBoundary creationDate) {
			this.creationDate = creationDate.getValue();

			return this;
		}

		public Builder setData(final String data) {
			this.data = data;

			return this;
		}

		public Builder setData(final Element data) {
			this.data = (data == null ? null : data.toXML());

			return this;
		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setEndDate(final Date endDate) {
			this.endDate = (endDate == null ? null : new DateTime(endDate));

			return this;
		}

		public Builder setEndDate(final DateTime endDate) {
			this.endDate = endDate;

			return this;
		}

		public Builder setEndDate(final DateBoundary endDate) {
			this.endDate = endDate.getValue();

			return this;
		}

		public Builder setEnrollmentId(final String enrollmentId) {
			this.enrollmentId = enrollmentId;

			return this;
		}

		public Builder setEnrollmentId(final Integer enrollmentId) {
			this.enrollmentId = (enrollmentId == null ? null : enrollmentId.toString());

			return this;
		}

		public Builder setEntityId(final String entityId) {
			this.entityId = entityId;

			return this;
		}

		public Builder setEntityId(final Integer entityId) {
			this.entityId = (entityId == null ? null : entityId.toString());

			return this;
		}

		public Builder setFlags(final Long flags) {
			this.flags = flags;

			return this;
		}

		public Builder setFlags(final RightFlag flags) {
			this.flags = flags.getValue();

			return this;
		}

		public Builder setGuid(final String guid) {
			this.guid = guid;

			return this;
		}

		public Builder setStatus(final EnrollmentStatus status) {
			this.status = status;

			return this;
		}

		public Builder setStatus(final Integer status) {
			this.status = (status == null ? null : EnrollmentStatus.valueOfValue(status));

			return this;
		}

		public Builder setStartDate(final Date startDate) {
			this.startDate = (startDate == null ? null : new DateTime(startDate));

			return this;
		}

		public Builder setStartDate(final DateTime startDate) {
			this.startDate = startDate;

			return this;
		}

		public Builder setStartDate(final DateBoundary startDate) {
			this.startDate = startDate.getValue();

			return this;
		}

		public Builder setReference(final String reference) {
			this.reference = reference;

			return this;
		}

		public Builder setSchema(final FlagSchema schema) {
			this.schema = schema;

			return this;
		}

		public Builder setSchema(final Integer schema) {
			this.schema = (schema == null ? null : FlagSchema.valueOfValue(schema));

			return this;
		}

		public Builder setUserId(final String userId) {
			this.userId = userId;

			return this;
		}

		public Builder setUserId(final Integer userId) {
			this.userId = (userId == null ? null : userId.toString());

			return this;
		}

		public Builder setVersion(final Integer version) {
			this.version = version;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public Enrollment build() {
			return new Enrollment(this);
		}
	}
}