package com.rockymadden.jbrainhoney.dlap.element;

import java.util.ArrayList;
import java.util.Collections;
import java.util.EnumSet;
import java.util.List;
import nu.xom.Attribute;
import nu.xom.Element;
import com.rockymadden.jbrainhoney.compare.NullComparatorUtility;
import com.rockymadden.jbrainhoney.dlap.enumeration.FlagSchema;
import com.rockymadden.jbrainhoney.dlap.enumeration.RightFlag;
import com.rockymadden.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/** A right. */
public final class Right implements XmlElement, Comparable<Right> {
	private static final long serialVersionUID = -547592649571120L;
	private final String actorId;
	private final String entityId;
	private final Long flags;
	private final FlagSchema schema;

	// Lazy attributes.
	private List<RightFlag> rightFlags;

	@SuppressWarnings("synthetic-access")
	private Right(final Builder builder) throws IllegalArgumentException {
		if(builder.actorId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.actorId))
			throw new IllegalArgumentException("Expected actorId to be valid.");
		else if(builder.entityId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.entityId))
			throw new IllegalArgumentException("Expected entityId to be valid.");

		actorId = builder.actorId;
		entityId = builder.entityId;
		flags = builder.flags;
		schema = builder.schema;
	}

	public Right(final nu.xom.Element element) throws IllegalArgumentException
	{
		if(!element.getQualifiedName().equals("rights")) // BH uses plural.
			throw new IllegalArgumentException("Expected right XML element.");
		else if(element.getAttribute("actorid") != null && element.getAttribute("actorid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("actorid").getValue()))

			throw new IllegalArgumentException("Expected actorId to be valid");
		else if(element.getAttribute("entityid") != null && element.getAttribute("entityid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("entityid").getValue()))

			throw new IllegalArgumentException("Expected entityId to be valid");

		actorId = (element.getAttribute("actorid") != null && element.getAttribute("actorid").getValue().length() > 0 ?
			element.getAttribute("actorid").getValue() : null);
		entityId = (element.getAttribute("entityid") != null && element.getAttribute("entityid").getValue().length() > 0 ?
			element.getAttribute("entityid").getValue() : null);
		flags = (element.getAttribute("flags") != null && element.getAttribute("flags").getValue().length() > 0 ?
			Long.parseLong(element.getAttribute("flags").getValue()) : null);
		schema = (element.getAttribute("schema") != null && element.getAttribute("schema").getValue().length() > 0 ?
			FlagSchema.valueOfValue(Integer.parseInt(element.getAttribute("schema").getValue())) : null);
	}

	@Override
	public int compareTo(final Right right) throws NullPointerException {
		if(right == null) throw new NullPointerException("Expected right to not be null.");

		int compared = NullComparatorUtility.compareIgnoreNullAndCase(this.actorId, right.getActorId());

		if(compared != 0) return compared;

		return NullComparatorUtility.compareIgnoreNullAndCase(this.entityId, right.getEntityId());
	}

	public String getActorId() {
		return actorId;
	}

	public String getEntityId() {
		return entityId;
	}

	public Long getFlags() {
		return flags;
	}

	public FlagSchema getSchema() {
		return schema;
	}

	// ** Convenience method. Returns null if flags is null. */
	public List<RightFlag> getRightFlags() {
		if(flags != null) {
			if(rightFlags == null) synchronized(this) {
				if(rightFlags == null)
					rightFlags = Collections.unmodifiableList(
						new ArrayList<RightFlag>(BitwiseUtility.toEnumeration(flags, EnumSet.allOf(RightFlag.class)))
					);
			}

			return rightFlags;
		}

		return null;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof Right)) return false;
		else return toString().equals(((Right)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(Right.class.getSimpleName() + "(");
		if(actorId != null) stringBuilder.append("actorId=" + actorId + ", ");
		if(entityId != null) stringBuilder.append("entityId=" + entityId + ", ");
		if(flags != null) stringBuilder.append("flags=" + flags.toString() + ", ");
		if(schema != null) stringBuilder.append("schema=" + schema.getValue().toString() + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("rights"); // BH uses plural.

		if(actorId != null) element.addAttribute(new Attribute("actorid", actorId));
		if(entityId != null) element.addAttribute(new Attribute("entityid", entityId));
		if(flags != null) element.addAttribute(new Attribute("flags", flags.toString()));
		if(schema != null) element.addAttribute(new Attribute("schema", schema.getValue().toString()));

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable Right elements. */
	public final static class Builder implements XmlElement.Builder {
		private String actorId;
		private String entityId;
		private Long flags;
		private FlagSchema schema;

		public Builder() {

		}

		public Builder setActorId(final String actorId) {
			this.actorId = actorId;

			return this;
		}

		public Builder setActorId(final Integer actorId) {
			this.actorId = (actorId == null ? null : actorId.toString());

			return this;
		}

		public Builder setEntityId(final String entityId) {
			this.entityId = entityId;

			return this;
		}

		public Builder setEntityId(final Integer entityId) {
			this.entityId = (entityId == null ? null : entityId.toString());

			return this;
		}

		public Builder setFlags(final Long flags) {
			this.flags = flags;

			return this;
		}

		public Builder setFlags(final RightFlag flags) {
			this.flags = flags.getValue();

			return this;
		}

		public Builder setSchema(final FlagSchema schema) {
			this.schema = schema;

			return this;
		}

		public Builder setSchema(final Integer schema) {
			this.schema = (schema == null ? null : FlagSchema.valueOfValue(schema));

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public Right build() {
			return new Right(this);
		}
	}
}