package com.rockymadden.jbrainhoney.network;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.List;

/**
 * Defines the interface of a RFC 3986 based Uri.
 *
 * @see <a href="http://tools.ietf.org/html/rfc3986">RFC 3986</a>
 */
public abstract interface Rfc3986Uri extends Uri {
	public abstract String getPath();

	public abstract String getFile();

	public abstract String getQuery();

	public abstract String getEscapedQuery();

	public abstract String getPercentEncodedQuery();

	public abstract List<Rfc3986Uri.QueryParameter> getQueryParameters();

	public abstract Rfc3986Uri.QueryParameter getQueryParameter(final int index);

	public abstract int getQueryParametersCount();

	public abstract String getFragment();

	/** Defines the interface of a Rfc3986Uri builder. */
	public static interface Builder extends Uri.Builder {
		public abstract Rfc3986Uri.Builder setPath(final String path);

		public abstract Rfc3986Uri.Builder setFile(final String file);

		public abstract Rfc3986Uri.Builder setQuery(final String query);

		public abstract Rfc3986Uri.Builder setQueryParameters(final List<QueryParameter> queryParameters);

		public abstract Rfc3986Uri.Builder addQueryParameter(final QueryParameter queryParameter);

		public abstract Rfc3986Uri.Builder setFragment(final String fragment);

		@Override
		public abstract Rfc3986Uri build();
	}

	/**
	 * A query parameter in a Rfc3986Uri.
	 *
	 * @see <a href="http://tools.ietf.org/html/rfc3986#section-3.4">RFC 3986 - Section 3.4</a>
	 */
	public final class QueryParameter extends AssignmentStringKeyValueParameter {
		private static final long serialVersionUID = -66572947201226301L;

		private QueryParameter(final Builder builder) {
			super(builder);
		}

		/** @return A string with ampersand and equals sign escaped. */
		@Override
		protected final String escape(final String escape) {
			return (escape != null ? escape.replace("=", "\\=").replace("&", "\\&") : null);
		}

		@Override
		protected final String unescape(final String unescape) {
			return (unescape != null ? unescape.replace("\\=", "=").replace("\\&", "&") : null);
		}

		/** @return A percent encoded string with spaces represented as percent twenty. */
		@Override
		protected final String percentEncode(final String percentEncode) throws NetworkException {
			try {
				return (percentEncode != null ? URLEncoder.encode(percentEncode, "UTF-8").replace("+", "%20") : null);
			} catch(UnsupportedEncodingException exception) {
				throw new NetworkException(exception);
			}
		}

		@Override
		protected final String percentDecode(final String percentDecode) throws NetworkException {
			try {
				return (percentDecode != null ? URLDecoder.decode(percentDecode.replace("%20", "+"), "UTF-8") : null);
			} catch(UnsupportedEncodingException exception) {
				throw new NetworkException(exception);
			}
		}

		/** A builder of immutable QueryParameter instances. */
		public final static class Builder extends AssignmentStringKeyValueParameter.Builder {
			public Builder(final String key, final String value) {
				super(key, value);
			}

			public Builder(final String key) {
				super(key);
			}

			@Override
			@SuppressWarnings("synthetic-access")
			public QueryParameter build() {
				return new QueryParameter(this);
			}
		}
	}
}
