package com.rockymadden.jbrainhoney.network;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.List;

/**
 * Defines the interface of a RFC 4516 based Uri.
 *
 * @see <a href="http://tools.ietf.org/html/rfc4516">RFC 4516</a>
 */
public abstract interface Rfc4516Uri extends Uri {
	public abstract String getDistinguishedName();

	public abstract String getEscapedDistinguishedName();

	public abstract String getPercentEncodedDistinguishedName();

	public abstract List<DistinguishedNameParameter> getDistinguishedNameParameters();

	public abstract DistinguishedNameParameter getDistinguishedNameParameter(final int index);

	public abstract int getDistinguishedNameParametersCount();

	public abstract String getAttributes();

	public abstract String getEscapedAttributes();

	public abstract String getPercentEncodedAttributes();

	public abstract List<AttributesParameter> getAttributesParameters();

	public abstract AttributesParameter getAttributesParameter(final int index);

	public abstract int getAttributesParametersCount();

	public abstract Scope getScope();

	public abstract String getFilter();

	/** Defines the interface of a Rfc4516Uri builder. */
	public static interface Builder extends Uri.Builder {
		public abstract Rfc4516Uri.Builder setDistinguishedName(final String distinguishedName);

		public abstract Rfc4516Uri.Builder setDistinguishedNameParameters(final List<DistinguishedNameParameter> distinguishedNameParameters);

		public abstract Rfc4516Uri.Builder addDistinguishedNameParameter(final DistinguishedNameParameter distinguishedNameParameter);

		public abstract Rfc4516Uri.Builder setAttributes(final String attributes);

		public abstract Rfc4516Uri.Builder setAttributesParameters(final List<AttributesParameter> attributesParameters);

		public abstract Rfc4516Uri.Builder addAttributesParameter(final AttributesParameter attributesParameter);

		public abstract Rfc4516Uri.Builder setScope(final Scope scope);

		public abstract Rfc4516Uri.Builder setFilter(final String filter);

		@Override
		public abstract Rfc4516Uri build();
	}

	/**
	 * A distinguished name parameter in a Rfc4516Uri.
	 *
	 * @see <a href="http://tools.ietf.org/html/rfc4516#section-2">RFC 4516 - Section 2</a>
	 */
	public final class DistinguishedNameParameter extends AssignmentStringKeyValueParameter {
		private static final long serialVersionUID = 194026659275836673L;

		private DistinguishedNameParameter(final Builder builder) {
			super(builder);
		}

		/**
		 * @return A string with backslash, comma, pound sign, plus sign, less than, greater than, semicolon,
		 * double quote, and equals sign escaped.
		 */
		@Override
		protected final String escape(final String escape) {
			return (escape != null ? escape.replace("\\", "\\\\").replace(",", "\\,").replace("#", "\\#")
				.replace("+", "\\+").replace("<", "\\<").replace(">", "\\>").replace(";", "\\;").replace("\"", "\\\"")
				.replace("=", "\\=") : null);
		}

		@Override
		protected final String unescape(final String unescape) {
			return (unescape != null ? unescape.replace("\\\\", "\\").replace("\\,", ",").replace("\\#", "#")
				.replace("\\+", "+").replace("\\<", "<").replace("\\>", ">").replace("\\;", ";").replace("\\\"", "\"")
				.replace("\\=", "=") : null);
		}

		/** @return A percent encoded string with spaces represented as percent twenty. */
		@Override
		protected final String percentEncode(final String percentEncode) throws NetworkException {
			try {
				return (percentEncode != null ? URLEncoder.encode(percentEncode, "UTF-8").replace("+", "%20") : null);
			} catch(UnsupportedEncodingException exception) {
				throw new NetworkException(exception);
			}
		}

		@Override
		protected final String percentDecode(final String percentDecode) throws NetworkException {
			try {
				return (percentDecode != null ? URLDecoder.decode(percentDecode.replace("%20", "+"), "UTF-8") : null);
			} catch(UnsupportedEncodingException exception) {
				throw new NetworkException(exception);
			}
		}

		/** A builder of immutable DistinguishedNameParameter instances. */
		public final static class Builder extends AssignmentStringKeyValueParameter.Builder {
			public Builder(final String key, final String value) {
				super(key, value);
			}

			public Builder(final String key) {
				super(key);
			}

			@Override
			@SuppressWarnings("synthetic-access")
			public DistinguishedNameParameter build() {
				return new DistinguishedNameParameter(this);
			}
		}
	}

	/**
	 * An attributes parameter in a Rfc4516Uri.
	 *
	 * @see <a href="http://tools.ietf.org/html/rfc4516#section-2">RFC 4516 - Section 2</a>
	 */
	public final class AttributesParameter extends AssignmentStringKeyValueParameter {
		private static final long serialVersionUID = 7866203719465923494L;

		private AttributesParameter(final Builder builder) {
			super(builder);
		}

		/** @return A string with comma and equals sign escaped. */
		@Override
		protected final String escape(final String escape) {
			return (escape != null ? escape.replace("=", "\\=").replace(",", "\\,") : null);
		}

		@Override
		protected final String unescape(final String unescape) {
			return (unescape != null ? unescape.replace("\\=", "=").replace("\\,", ",") : null);
		}

		/** @return A percent encoded string with spaces represented as percent twenty. */
		@Override
		protected final String percentEncode(final String percentEncode) throws NetworkException {
			try {
				return (percentEncode != null ? URLEncoder.encode(percentEncode, "UTF-8").replace("+", "%20") : null);
			} catch(UnsupportedEncodingException exception) {
				throw new NetworkException(exception);
			}
		}

		@Override
		protected final String percentDecode(final String percentDecode) throws NetworkException {
			try {
				return (percentDecode != null ? URLDecoder.decode(percentDecode.replace("%20", "+"), "UTF-8") : null);
			} catch(UnsupportedEncodingException exception) {
				throw new NetworkException(exception);
			}
		}

		/** A builder of immutable AttributesParameter instances. */
		public final static class Builder extends AssignmentStringKeyValueParameter.Builder {
			public Builder(final String key, final String value) {
				super(key, value);
			}

			public Builder(final String key) {
				super(key);
			}

			@Override
			@SuppressWarnings("synthetic-access")
			public AttributesParameter build() {
				return new AttributesParameter(this);
			}
		}
	}

	/**
	 * Enumerates Rfc4516Uri scopes.
	 *
	 * @see <a href="http://tools.ietf.org/html/rfc4516#section-2">RFC 4516 - Section 2</a>
	 */
	public enum Scope {
		BASE,
		ONE,
		SUB
	}
}
