package com.rockymadden.jbrainhoney.network.http;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import com.rockymadden.jbrainhoney.network.CommonInternetUri;

/** An implementation of a HTTP based CommonInternetUri. */
public final class HttpUri implements CommonInternetUri {
	private static final long serialVersionUID = -725395729563L;
	private final Scheme scheme;
	private final String username;
	private final String password;
	private final String hostname;
	private final Integer port;
	private final String path;
	private final String file;
	private final List<QueryParameter> queryParameters;
	private final String fragment;

	@SuppressWarnings("synthetic-access")
	private HttpUri(final Builder builder) throws IllegalArgumentException {
		if(builder.scheme == null) throw new IllegalArgumentException("Expected scheme to not be null.");
		else if(builder.hostname == null) throw new IllegalArgumentException("Expected hostname to not be null.");
		else if(builder.queryParameters == null)
			throw new IllegalArgumentException("Expected queryParameters to not be null.");
		else if(builder.port != null && (builder.port < 0 || builder.port > 65535))
			throw new IllegalArgumentException("Expected port to have a value of 0 to 65535.");

		scheme = builder.scheme;
		username = builder.username;
		password = builder.password;
		hostname = builder.hostname;
		port = builder.port;
		path = builder.path;
		file = builder.file;
		queryParameters = Collections.unmodifiableList(new ArrayList<QueryParameter>(builder.queryParameters));
		fragment = builder.fragment;
	}

	@Override
	public Scheme getScheme() {
		return scheme;
	}

	@Override
	public String getUsername() {
		return username;
	}

	@Override
	public String getPassword() {
		return password;
	}

	@Override
	public String getHostname() {
		return hostname;
	}

	@Override
	public Integer getPort() {
		return port;
	}

	@Override
	public String getPath() {
		return path;
	}

	@Override
	public String getFile() {
		return file;
	}

	@Override
	public String getQuery() {
		StringBuilder query = new StringBuilder();
		QueryParameter queryParameter = null;

		for(int i = 0; i < queryParameters.size(); i++) {
			queryParameter = queryParameters.get(i);

			if(i == 0) query.append(queryParameter.toString());
			else query.append("&" + queryParameter.toString());
		}

		return query.toString();
	}

	@Override
	public String getEscapedQuery() {
		StringBuilder query = new StringBuilder();
		QueryParameter queryParameter = null;

		for(int i = 0; i < queryParameters.size(); i++) {
			queryParameter = queryParameters.get(i);

			if(i == 0) query.append(queryParameter.toEscapedString());
			else query.append("&" + queryParameter.toEscapedString());
		}

		return query.toString();
	}

	@Override
	public String getPercentEncodedQuery() {
		StringBuilder query = new StringBuilder();
		QueryParameter queryParameter = null;

		for(int i = 0; i < queryParameters.size(); i++) {
			queryParameter = queryParameters.get(i);

			if(i == 0) query.append(queryParameter.toPercentEncodedString());
			else query.append("&" + queryParameter.toPercentEncodedString());
		}

		return query.toString();
	}

	@Override
	public List<QueryParameter> getQueryParameters() {
		return queryParameters;
	}

	@Override
	public QueryParameter getQueryParameter(final int index) {
		if(index >= 0 && index <= (queryParameters.size() - 1)) return queryParameters.get(index);

		return null;
	}

	@Override
	public int getQueryParametersCount() {
		return queryParameters.size();
	}

	@Override
	public String getFragment() {
		return fragment;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof HttpUri)) return false;

		return toString().equals(((HttpUri)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	/** @return A URI with all queryParameters percent encoded. */
	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();
		QueryParameter queryParameter = null;

		stringBuilder.append(scheme.toString().toLowerCase() + "://");

		if(username != null) {
			stringBuilder.append(username);

			if(password != null) stringBuilder.append(":" + password + "@");
			else stringBuilder.append("@");
		}

		stringBuilder.append(hostname);

		if(port != null) stringBuilder.append(":" + port);

		stringBuilder.append("/");

		if(path != null)
			if(path.substring(0, 1).equals("/") && path.substring(path.length() - 1, path.length()).equals("/"))
				stringBuilder.append(path.substring(1, path.length()));
			else if(path.substring(0, 1).equals("/") && !path.substring(path.length() - 1, path.length()).equals("/"))
				stringBuilder.append(path.substring(1, path.length()) + "/");
			else if(!path.substring(0, 1).equals("/") && path.substring(path.length() - 1, path.length()).equals("/"))
				stringBuilder.append(path);
			else stringBuilder.append(path + "/");

		if(file != null) stringBuilder.append(file);

		if(queryParameters.size() > 0) {
			stringBuilder.append("?");

			for(int i = 0; i < queryParameters.size(); i++) {
				queryParameter = queryParameters.get(i);

				if(i == 0) stringBuilder.append(queryParameter.toPercentEncodedString());
				else stringBuilder.append("&" + queryParameter.toPercentEncodedString());
			}
		}

		if(fragment != null) stringBuilder.append("#" + fragment);

		return stringBuilder.toString();
	}

	/** A builder of immutable HttpUri instances. */
	public final static class Builder implements CommonInternetUri.Builder {
		private final Scheme scheme;
		private String username;
		private String password;
		private final String hostname;
		private Integer port;
		private String path;
		private String file;
		private List<QueryParameter> queryParameters;
		private String fragment;

		public Builder(final Scheme scheme, final String hostname) {
			this.hostname = hostname;
			this.scheme = scheme;
			this.queryParameters = Collections.synchronizedList(new ArrayList<QueryParameter>());
		}

		@Override
		public Builder setUsername(final String username) {
			this.username = username;

			return this;
		}

		@Override
		public Builder setPassword(final String password) {
			this.password = password;

			return this;
		}

		@Override
		public Builder setPort(final Integer port) {
			this.port = port;

			return this;
		}

		@Override
		public Builder setPath(final String path) {
			this.path = path;

			return this;
		}

		@Override
		public Builder setFile(final String file) {
			this.file = file;

			return this;
		}

		@Override
		public Builder setQuery(final String query) throws HttpException {
			String mutableQuery = null;
			String[] parameters = null;
			String[] pair = null;

			queryParameters.clear();

			try {
				mutableQuery = URLDecoder.decode(query.replace("%20", "+"), "UTF-8");
				parameters = mutableQuery.split("(?<!\\\\)&");

				for(int i = 0; i < parameters.length; i++) {
					pair = parameters[i].split("(?<!\\\\)=");

					if(pair.length == 1) queryParameters.add(new QueryParameter.Builder(pair[0], null).build());
					else if(pair.length == 2) queryParameters.add(new QueryParameter.Builder(pair[0], pair[1]).build());
				}
			} catch(UnsupportedEncodingException exception) {
				throw new HttpException(exception);
			}

			return this;
		}

		@Override
		public Builder setQueryParameters(final List<QueryParameter> queryParameters) {
			this.queryParameters = Collections.synchronizedList(new ArrayList<QueryParameter>(queryParameters));

			return this;
		}

		@Override
		public Builder addQueryParameter(final QueryParameter queryParameter) {
			if(queryParameter != null) queryParameters.add(queryParameter);

			return this;
		}

		@Override
		public Builder setFragment(final String fragment) {
			this.fragment = fragment;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public HttpUri build() {
			return new HttpUri(this);
		}
	}

	/** Enumerates schemes in a HttpUri. */
	public enum Scheme implements Schemeable {
		HTTP,
		HTTPS
	}
}
